#include <SFML/Window.hpp>
#include "Core.hpp"
#include "Log.hpp"
#include "Animation.hpp"
#include "Asset.hpp"
#include "StateSplash.hpp"
#include "StateIngame.hpp"

bool ready = false;

Core::Core(sf::RenderWindow* window) :
m_renderer(window),
m_state(new StateSplash(StateSplash::SplashType::MAIN)),
m_running(true),  
m_window(window) {
	m_renderer.sort(); 
}
  
Core::~Core() {
	
}

void Core::run() { 
	sf::Clock clock; 
	float ft;
	float minimum = 1.f / 30.f;
	Log(INFO, "Launching core.");
	while(m_running) {
		events();
		ft = clock.restart().asSeconds();
		if (!ready && m_state->isReady()) {
			ready = true;
			ft = 0.f;
		}
		if (ready) {
			while (ft > minimum) {
				ft -= minimum;
				update(minimum);
			}
			update(ft);
		}
		if (m_running && ready) {
			m_renderer.render();
		}
	}
	Log(INFO, "Closing core.");
}

/*
void Core::registerEntity(IEntity* entity) {
	m_entities.push_back(entity);
}

void Core::deregisterEntity(IEntity* entity) {
	m_entities.erase(std::remove_if(m_entities.begin(), m_entities.end(), [entity](IEntity* elem) { 
		return elem == entity;
	}), m_entities.end());
}*/

void Core::events() {
	sf::Event event;

	while(m_window->pollEvent(event)) {
		if(event.type == sf::Event::Closed || (event.type == sf::Event::KeyReleased && event.key.code == sf::Keyboard::Escape)) {
			m_running = false;
		}

		m_state->onEvent(event);
	}
}

void Core::update(float dt) {
	m_state->onUpdate(dt);
	if (m_oldState) {
		m_oldState->onUpdate(dt);
	}
	m_renderer.update(dt);

	if (m_state->isFin()) {
		m_oldState = m_state;
		State* state = m_state->next();
		if (!state) {
			m_running = false;
		}
		else {
			m_state = state;
		}
		ready = false;
	}
	if (m_oldState && m_oldState->isDead()) {
		delete m_oldState;
		m_oldState = nullptr;
	}
}