#include "Spectre.hpp"
#include "Renderer.hpp"
#include "Asset.hpp"
#include "Log.hpp"
#include "Player.hpp"
#include "Alermath.hpp"

float Spectre::s_chaos = 0.f;
sf::Vector2f Spectre::s_limit(0.f, 0.f);
Player* Spectre::s_player = nullptr;

Spectre::Spectre(Spectre::Settings settings): m_settings(settings) {
	Renderer::registerDrawable(this, 10000, Renderer::Mode::FG, "Spectre " + m_settings.sheet);
	Renderer::sort();

	for (unsigned i = 0; i < m_settings.frames; ++i) {
		m_sprites.emplace_back(false, sf::Sprite());
		m_sprites.back().second.setTexture(Asset::texture(m_settings.sheet));
		m_sprites.back().second.setTextureRect(sf::IntRect{
			0,
			// msvc is fucking autistic just like me
			// don't be a retard like me
			// don't use msvc like me
			int(Asset::texture(m_settings.sheet).getSize().y / m_settings.frames * i),
			int(Asset::texture(m_settings.sheet).getSize().x),
			int(Asset::texture(m_settings.sheet).getSize().y / m_settings.frames)
		});
		m_sprites.back().second.setOrigin(sf::Vector2f(Asset::texture(m_settings.sheet).getSize().x / 2u, Asset::texture(m_settings.sheet).getSize().y / m_settings.frames / 2u));
		m_sprites.back().second.setScale(sf::Vector2f(m_settings.size, m_settings.size));
		//m_sprites.back().second.setColor(sf::Color(255, 255, 255, 255));
		m_sprites.back().second.setColor(sf::Color(255, 255, 255, 0));
	}
	m_sprites[0].first = true;

	m_sound.setBuffer(Asset::sound("data/spectre/" + std::to_string(rand() % 1 + 1) + ".ogg"));
	m_sound.setRelativeToListener(true);
	m_sound.setLoop(true);
	m_sound.play();
}

Spectre::~Spectre() {
	Renderer::deregisterDrawable(this);
}

void Spectre::update(float dt) {
	m_clock += dt;
	m_total_clock += dt;

	if (m_total_clock < m_settings.duration) {
		if (m_alpha < 255.f) {
			m_alpha += dt * 100.f;
			if (m_alpha > 255.f) {
				m_alpha = 255.f;
			}

			for (auto& spr : m_sprites) {
				sf::Color c = spr.second.getColor();
				spr.second.setColor(sf::Color(c.r, c.g, c.b, m_alpha));
			}
		}
	}
	else {
		m_alpha -= dt * 100.f;
		for (auto& spr : m_sprites) {
			sf::Color c = spr.second.getColor();
			spr.second.setColor(sf::Color(c.r, c.g, c.b, m_alpha));
		}
		if (m_alpha < 10.f) {
			m_dead = true;
		}
	}

	for (auto& spr : m_sprites) {
		sf::Color c = spr.second.getColor();
		spr.second.setColor(sf::Color(c.r, c.g, c.b, m_alpha * getChaos()));
	}

	if (m_clock >= m_settings.frameDuration) {
		m_clock = 0.f;
		m_frame++;
		m_frame %= m_settings.frames;

		for (size_t i = 0; i < m_sprites.size(); ++i) {
			m_sprites[i].first = false;
		}
		m_sprites[m_frame].first = true;

		if (s_chaos > CHAOS_OVERLAY_THRESHOLD) {
			for (int i = 0; i < m_sprites.size() - 1; ++i) {
				if (rand() % 100 > 50 * s_chaos) {
					m_sprites[(m_frame - i) % m_settings.frames].first = true;
				}
			}
		}
	}

	float q;
	q = CHAOS_SCALE_THRESHOLD;
	if (s_chaos > q && m_settings.scale) {
		float intensity = 100.f - (s_chaos - q) / (100.f - q);

		if (m_scale_clock.getElapsedTime().asSeconds() > 1.f) {
			float d = 0.f;
			d = float(rand() % 101); // 0~100
			d /= 100.f; // 0~1
			d *= intensity / 25.f; // 0~4
			setScale(1.f + d, 1.f + d);
			m_scale_clock.restart();
		}
	}
	else {
		setScale(1.f, 1.f);
	}

	if (m_settings.follow) {
		float x1 = s_player->getPosition().x;
		float x2 = getPosition().x;
		float y1 = s_player->getPosition().y;
		float y2 = getPosition().y;

		float disttttt = math::dist(getPosition(), s_player->getPosition());
		const float camera_speed = 50.f * (disttttt > 800 ? 60.f : 1.f) + (Spectre::getChaos() * 35.f);

		if (x1 != x2 || y1 != y2) {
			if (std::fabs(x2 - x1) > dt * camera_speed) {
				move(dt * camera_speed * math::sgn(x1 - x2), 0);
			}
			else {
				move(x2 - x1, 0);
			}
			if (std::fabs(y2 - y1) > dt * camera_speed) {
				move(0, dt * camera_speed * math::sgn(y1 - y2));
			}
			else {
				move(0, y2 - y1);
			}
		}

		x2 = getPosition().x - x1;
		y2 = getPosition().y - y1;

		float dist = std::sqrt(x2 * x2 + y2 * y2);

		//Log(DEBUG, dist, " ", x2, " ", y2, " ", (500.f - dist) / 5.f);

		/*static float pos = 0.f;
		static bool bounce = false;
		if (!bounce) {
			pos -= dt;
			if (pos < -10.f) {
				pos = -10.f;
				bounce = true;
			}
		}
		else {
			pos += dt;
			if (pos > 10.f) {
				pos = 10.f;
				bounce = false;
			}
		}
		Log(DEBUG, pos);*/
		m_sound.setPosition(x2 / 100.f, 0, y2 / 100.f);
		//m_sound.setVolume((500.f - dist) / 5.f);
	}
	else {
		const float camera_speed = 50.f + (Spectre::getChaos() * 8.f);
		if (m_m1 == -100.f) {
			m_m1 = 0.f;
			m_m2 = float(rand() % 360) / 180.f * 3.1415f;
		}

		m_m1 += dt;
		float x = dt * 2.f * m_m1 * camera_speed;
		float y = dt * 2.f * m_m1 * camera_speed;

		move(x * std::cos(m_m2), y * std::cos(m_m2));

		m_sound.setPosition((getPosition().x - s_player->getPosition().x) / 100.f, 0, (getPosition().y - s_player->getPosition().y) / 100.f);

	}

	if (m_settings.rotate) {
		float dx = getPosition().x - s_player->getPosition().x;
		float dy = getPosition().y - s_player->getPosition().y;
		float dist = std::sqrt(dx * dx + dy * dy);

		//Log(DEBUG, std::atan2(dy, dx) * 180.f / 3.1415f);
		setRotation(std::atan2(dy, dx) * 180.f / 3.1415f);
	}
	if (m_settings.rotateRandom) {
		//Log(DEBUG, std::atan2(dy, dx) * 180.f / 3.1415f);
		rotate(3.f * dt * float(rand() % 200 - 50));
	}

	if (m_settings.skew) {
		setScale(getScale().x + (rand() % 21 - 10) / 100.f * getChaos() * getChaos(), getScale().y + (rand() % 21 - 10) / 100.f * getChaos() * getChaos());
	}

	if (m_settings.rainbow) {
		for (int i = 0; i < 2;++i) {
			if (!m_rdir[i]) {
				m_rdir[i] = rand() % 2 ? 1 : -1;
			}
			if (m_rdir[i] == 1) {
				m_rai[i] -= dt * 255.f * getChaos();
				if (m_rai[i] < 0.f || rand() % 50 == 0) {
					m_rdir[i] = -1;
				}
			}
			else {
				m_rai[i] += dt * 255.f * getChaos();
				if (m_rai[i] > 255.f || rand() % 50 == 0) {
					m_rdir[i] = 1;
				}
			}
		}
		for (auto& spr : m_sprites) {
			sf::Color c = spr.second.getColor();
			spr.second.setColor({ sf::Uint8(m_rdir[0]), sf::Uint8(m_rdir[1]), sf::Uint8(255.f - float(m_rdir[0] + m_rdir[1]) / 2.f), c.a });
		}
	}

	m_announceClock += dt;
	float dist = math::dist(getPosition(), s_player->getPosition());
	//Log(DEBUG, dist, " ", getPosition().y, " ", s_player->getPosition().y);
	if (dist < 300.f && m_announceClock >= 0.1f) {
		s_player->announce(dist);
		if (dist <= 10.f) {
			//Log(DEBUG, "Too close");
			addChaos(0.01f / 10.f);
		}
		else if(dist < m_settings.scareRange) {
			float val = (m_settings.scareRange - dist - 10.f) / ((m_settings.scareRange - 10.f) * 10.f);
			addChaos(val / 10.f);
			//Log(DEBUG, val, " ", dist);
		}
		else {
			//Log(DEBUG, "Too far");
		}
		m_announceClock = 0.f;
	}
	if (dist > 1400.f) {
		//m_dead = true;
	}
}

void Spectre::draw(sf::RenderTarget& rt, sf::RenderStates states) const {
	states.transform *= getTransform();
	for (const auto& sprite: m_sprites) {
		if (sprite.first) {
			rt.draw(sprite.second, states);
		}
	}
}

void Spectre::setChaos(float chaos) {
	s_chaos = chaos;
}

void Spectre::addChaos(float chaos) {
	s_chaos += chaos;
	if (s_chaos > s_limit.y) {
		s_chaos = s_limit.y;
	}
	if (s_chaos < s_limit.x) {
		s_chaos = s_limit.x;
	}
}

float Spectre::getChaos() {
	return s_chaos;
}
void Spectre::setPlayer(Player* player) {
	s_player = player;
}