#include "Voice.hpp"
#include "Asset.hpp"
#include "Renderer.hpp"
#include "Spectre.hpp"

Voice::Voice() {
	m_sound.setLoop(false);
	m_sound.setVolume(95.f);
	m_text.setCharacterSize(32);
	m_text.setOutlineColor(sf::Color::Black);
	m_text.setOutlineThickness(3);
	m_text.setFont(Asset::font("data/Scratch.ttf"));
	m_text.setPosition(640, 640);
	Renderer::registerDrawable(&m_text, 10000, Renderer::Mode::UI);
}

Voice::~Voice() {
	Renderer::deregisterDrawable(&m_text);
}

void Voice::addLine(std::string filename, float timer, bool pitch) {
	Line l;
	l.clock = 0.f;
	l.timer = timer;
	l.filename = filename;
	l.pitch = pitch;
	m_lines.push(l);
	Asset::sound(filename);
}

void Voice::addSubtitle(float timer, float duration, std::string text, sf::Color color) {
	Sub s;
	s.clock = 0.f;
	s.timer = timer;
	s.duration = duration;
	s.color = color;
	s.text = text;
	m_subs.push(s);
}

void Voice::update(float dt) {
	if (!m_lines.empty()) {
		Line& line = m_lines.front();
		line.clock += dt;
		if (!line.played && m_sound.getStatus() != sf::Sound::Playing && line.clock > line.timer) {
			m_sound.setBuffer(Asset::sound(line.filename));
			Log(INFO, "Playing voice ", line.filename);
			if (line.pitch) {
				m_sound.setPitch(1.f + ((rand() % 10) / 50.f) * Spectre::getChaos());
			}
			else {
				m_sound.setPitch(1.f);
			}
			m_sound.play();
			line.played = true;
		}
		if (m_sound.getStatus() == sf::Sound::Stopped && line.played) {
			Log(INFO, "Popping voice ", line.filename);
			m_lines.pop();
		}
	}
	if (!m_subs.empty()) {
		Sub& sub = m_subs.front();
		sub.clock += dt;
		if (sub.clock >= sub.timer) {
			m_text.setString(sub.text);
			m_text.setFillColor(sub.color);
			m_text.setOrigin(m_text.getLocalBounds().width / 2, m_text.getLocalBounds().height / 2);
		}
		if (sub.clock >= sub.timer + sub.duration) {
			m_text.setString("");
			m_subs.pop();
		}
	}
}